/*
 * 作成開始:2022年8月9日
 * 染色装置:J22-15
 * NCU:ArduinoMega I2C_SDA_SCL 
 * SteppingDriver:PololuTic36v4
 * SteppingMotoer:
 * pulse計算
 * 1pulse : 1.8° = x : 360° ⇄ x = 200pulse/rot
 * 1/8pulseにすると x = 1600pulse/rot
 * ﾎﾞｰﾙﾈｼﾞ:1rotで10mm → 1600/10mm → 160step/mm 
 * request velocity 45mm/sec　→　160step/mm × 45mm/sec = 7200step/sec
 * ﾘﾐｯﾄｾﾝｻｰが反応する位置をHOMEと称する
 * HOMEから少し動いて、ｾﾝｻｰﾗﾝﾌﾟが消える位置を原点(Origin)と称する
 */
 
//*****I2C library*****//
#include <Wire.h>

//*****Nextion library*****//
#include <EasyNextionLibrary.h>
EasyNex myNex(Serial1);

//*****Pololu library*****//
#include <Tic.h>
TicI2C ticX(14); //x axis address
TicI2C ticY(15); //y axis address
TicI2C ticZ(16); //z axis address

// ***** Position set *****//
int32_t PosX;
int32_t Pos16Y; //16000
int32_t Pos15Y; //15000
int32_t Pos14Y; //14000
int32_t Pos12Y; //12000
int32_t PosZ;

// ***** Speed & Accel set *****//
int32_t SpeedX;
int32_t SpeedY;
int32_t SpeedZ;
int32_t AccelX;
int32_t AccelY;
int32_t AccelZ;

// ***** Flag set *****//
int StopFlag = 0;
int ResetFlag = 0;
int PauseFlag = 0;
int CheckFlag = 0;
int targetPosition = 0;

//----------------//
//+++++set up+++++//
//----------------//

void setup()
{
  // ***** I2C Initialization *****//
  Wire.begin();
  Serial.begin(19200);
  myNex.begin(9600);
  delay(2000);
  myNex.writeStr("page 0");
  
  // ***** TicReset *****//
  ticX.reset();
  delay(10);
  ticY.reset();
  delay(10);
  ticZ.reset();
  delay(10);

  // ***** Tic velocity & accelerationset *****//
  SpeedX = 72000000;
  SpeedY = 72000000;
  SpeedZ = 72000000;
  ticX.setMaxSpeed(SpeedX); //vx = 45mm/sec(155mm/3.4sec)
  delay(10);
  ticY.setMaxSpeed(SpeedY); //vy = 45mm/sec(155mm/3.4sec)
  delay(10);
  ticZ.setMaxSpeed(SpeedZ); //vz = 45mm/sec(155mm/3.4sec)
  delay(10);
  AccelX = 1500000;
  AccelY = 1500000;
  AccelZ = 1500000;
  ticX.setMaxAccel(AccelX);
  delay(10);
  ticY.setMaxAccel(AccelY);
  delay(10);
  ticZ.setMaxAccel(AccelZ);
  delay(10);

  // ***** Tic exitSafeStart *****//
  ticX.exitSafeStart();
  delay(10);
  ticY.exitSafeStart();
  delay(10);
  ticZ.exitSafeStart();
  delay(10);

  // ***** Tic driverPowerON *****//
  ticX.energize();
  delay(10);
  ticY.energize();
  delay(10);
  ticZ.energize();
  delay(10);
}

//--------------------------------//
// +++ Reset Command Time Out +++ //
//--------------------------------// 
void resetCommandTimeout()
{
  ticX.resetCommandTimeout();
  delay(10);
  ticY.resetCommandTimeout();
  delay(10);
  ticZ.resetCommandTimeout();
  delay(10);
}

//----------------------//
// +++ Main routine +++ //
//----------------------//
void loop()
{
   myNex.NextionListen();
}

//----------------------//
// +++ Waiting Time +++ //
//----------------------//
void delayWhileResettingCommandTimeout(uint32_t ms)
{
  uint32_t start = millis();
  do
  {
    resetCommandTimeout();
  } while ((uint32_t)(millis() - start) <= ms);
}

//------------------------------------//
// +++ pololu Target Position Set +++ //
//------------------------------------//
void XpololuTargetPositionVer1()
{
  PosX = 5000;
  ticX.setTargetPosition(PosX);
  targetPosition = PosX;
  X_waitForPosition(targetPosition);
  delayWhileResettingCommandTimeout(2000);
}

void ZpololuTargetPositionVer1()
{
  PosZ = 7000;
  ticZ.setTargetPosition(PosZ);
  targetPosition = PosZ;
  Z_waitForPosition(targetPosition);
  delayWhileResettingCommandTimeout(2000);
}

void YpololuTargetPositionVer1()
{
  Pos16Y = 16000;
  Pos12Y = 12000;
  ticY.setTargetPosition(Pos16Y);
  targetPosition = Pos16Y;
  Y_waitForPosition(targetPosition);
  delayWhileResettingCommandTimeout(1000);
  ticY.setTargetPosition(Pos12Y);
  targetPosition = Pos12Y;
  Y_waitForPosition(targetPosition);
  ticY.setTargetPosition(Pos16Y);
  targetPosition = Pos16Y;
  Y_waitForPosition(targetPosition);
  delayWhileResettingCommandTimeout(1000);
  ticY.setTargetPosition(Pos12Y);
  targetPosition = Pos12Y;
  Y_waitForPosition(targetPosition);
  ticY.setTargetPosition(Pos16Y);
  targetPosition = Pos16Y;
  Y_waitForPosition(targetPosition);
  delayWhileResettingCommandTimeout(1000);
  ticY.setTargetPosition(Pos12Y);
  targetPosition = Pos12Y;
  Y_waitForPosition(targetPosition);
  ticY.setTargetPosition(Pos16Y);
  targetPosition = Pos16Y;
  Y_waitForPosition(targetPosition);
  delayWhileResettingCommandTimeout(1000);
  ticY.setTargetPosition(Pos12Y);
  targetPosition = Pos12Y;
  Y_waitForPosition(targetPosition);
  ticY.setTargetPosition(Pos16Y);
  targetPosition = Pos16Y;
  Y_waitForPosition(targetPosition);
  delayWhileResettingCommandTimeout(1000);
  ticY.setTargetPosition(0);
  targetPosition = 0;
  Y_waitForPosition(targetPosition);
}

void ZpololuZeroPosition()
{
  ticZ.setTargetPosition(0);
  targetPosition = 0;
  Z_waitForPosition(targetPosition);
  delayWhileResettingCommandTimeout(2000);
}

void XpololuTargetPositionVer2()
{
  PosX = 10000;
  ticX.setTargetPosition(PosX);
  targetPosition = PosX;
  X_waitForPosition(targetPosition);
  delayWhileResettingCommandTimeout(2000);
}

void YpololuTargetPositionVer2()
{
  Pos16Y = 16000;
  Pos12Y = 12000;
  ticY.setTargetPosition(Pos16Y);
  targetPosition = Pos16Y;
  Y_waitForPosition(targetPosition);
  delayWhileResettingCommandTimeout(500);
  ticY.setTargetPosition(Pos12Y);
  targetPosition = Pos12Y;
  Y_waitForPosition(targetPosition);
  ticY.setTargetPosition(Pos16Y);
  targetPosition = Pos16Y;
  Y_waitForPosition(targetPosition);
  delayWhileResettingCommandTimeout(500);
  ticY.setTargetPosition(Pos12Y);
  targetPosition = Pos12Y;
  Y_waitForPosition(targetPosition);
  ticY.setTargetPosition(Pos16Y);
  targetPosition = Pos16Y;
  Y_waitForPosition(targetPosition);
  delayWhileResettingCommandTimeout(500);
  ticY.setTargetPosition(Pos12Y);
  targetPosition = Pos12Y;
  Y_waitForPosition(targetPosition);
  ticY.setTargetPosition(Pos16Y);
  targetPosition = Pos16Y;
  Y_waitForPosition(targetPosition);
  delayWhileResettingCommandTimeout(500);
  ticY.setTargetPosition(Pos12Y);
  targetPosition = Pos12Y;
  Y_waitForPosition(targetPosition);
  ticY.setTargetPosition(Pos16Y);
  targetPosition = Pos16Y;
  Y_waitForPosition(targetPosition);
  delayWhileResettingCommandTimeout(500);
  ticY.setTargetPosition(0);
  targetPosition = 0;
  Y_waitForPosition(targetPosition);
}

void XpololuTargetPositionVer3()
{
  PosX = 15000;
  ticX.setTargetPosition(PosX);
  targetPosition = PosX;
  X_waitForPosition(targetPosition);
  delayWhileResettingCommandTimeout(2000);
}

void YpololuTargetPositionVer3()
{
  Pos16Y = 16000;
  Pos14Y = 14000;
  ticY.setTargetPosition(Pos16Y);
  targetPosition = Pos16Y;
  Y_waitForPosition(targetPosition);
  delayWhileResettingCommandTimeout(500);
  ticY.setTargetPosition(Pos14Y);
  targetPosition = Pos14Y;
  Y_waitForPosition(targetPosition);
  ticY.setTargetPosition(Pos16Y);
  targetPosition = Pos16Y;
  Y_waitForPosition(targetPosition);
  delayWhileResettingCommandTimeout(500);
  ticY.setTargetPosition(Pos14Y);
  targetPosition = Pos14Y;
  Y_waitForPosition(targetPosition);
  ticY.setTargetPosition(Pos16Y);
  targetPosition = Pos16Y;
  Y_waitForPosition(targetPosition);
  delayWhileResettingCommandTimeout(500);
  ticY.setTargetPosition(Pos14Y);
  targetPosition = Pos14Y;
  Y_waitForPosition(targetPosition);
  ticY.setTargetPosition(Pos16Y);
  targetPosition = Pos16Y;
  Y_waitForPosition(targetPosition);
  delayWhileResettingCommandTimeout(500);
  ticY.setTargetPosition(Pos14Y);
  targetPosition = Pos14Y;
  Y_waitForPosition(targetPosition);
  ticY.setTargetPosition(Pos16Y);
  targetPosition = Pos16Y;
  Y_waitForPosition(targetPosition);
  delayWhileResettingCommandTimeout(500);
  ticY.setTargetPosition(0);
  targetPosition = 0;
  Y_waitForPosition(targetPosition);
}

void XpololuTargetPositionVer4()
{
  PosX = 20000;
  ticX.setTargetPosition(PosX);
  X_waitForPosition(PosX);
  delayWhileResettingCommandTimeout(2000);
}

void YpololuTargetPositionVer4()
{
  Pos16Y = 16000;
  Pos15Y = 15000;
  ticY.setTargetPosition(Pos16Y);
  targetPosition = Pos16Y;
  Y_waitForPosition(targetPosition);
  delayWhileResettingCommandTimeout(500);
  ticY.setTargetPosition(Pos15Y);
  targetPosition = Pos15Y;
  Y_waitForPosition(targetPosition);
  ticY.setTargetPosition(Pos16Y);
  Y_waitForPosition(Pos16Y);
  delayWhileResettingCommandTimeout(500);
  ticY.setTargetPosition(Pos15Y);
  Y_waitForPosition(Pos15Y);
  ticY.setTargetPosition(Pos16Y);
  Y_waitForPosition(Pos16Y);
  delayWhileResettingCommandTimeout(500);
  ticY.setTargetPosition(Pos15Y);
  Y_waitForPosition(Pos15Y);
  ticY.setTargetPosition(Pos16Y);
  Y_waitForPosition(Pos16Y);
  delayWhileResettingCommandTimeout(500);
  ticY.setTargetPosition(Pos15Y);
  Y_waitForPosition(Pos15Y);
  ticY.setTargetPosition(Pos16Y);
  Y_waitForPosition(Pos16Y);
  delayWhileResettingCommandTimeout(500);
  ticY.setTargetPosition(0);
  Y_waitForPosition(0);
}

//---------------------//
// +++ pololu STOP +++ //
//---------------------//
void pololuStop()
{
  SpeedX = 0;
  SpeedY = 0;
  SpeedZ = 0;
  ticX.setTargetVelocity(SpeedX);
  delay(10);
  ticY.setTargetVelocity(SpeedY);
  delay(10);
  ticZ.setTargetVelocity(SpeedZ);
  delay(10);
}

//----------------------------------//
// +++ pololu Go Reset Position +++ //
//----------------------------------//
void pololuGoResetPosition()
{
  ticZ.setTargetPosition(0);
  Z_waitForPosition(0);
  delayWhileResettingCommandTimeout(500);
  ticY.setTargetPosition(0);
  Y_waitForPosition(0);
  delayWhileResettingCommandTimeout(500);
  PosX = 51000;
  ticX.setTargetPosition(PosX);
  X_waitForPosition(PosX);
  delayWhileResettingCommandTimeout(500);
}

//------------------------//
// +++ pololu Go HOME +++ //
//------------------------//
void XpololuGoHome()
{
  ticX.goHomeReverse();
  do
  {
  } while (ticX.getHomingActive() == 1);
}

void YpololuGoHome()
{
  ticY.goHomeReverse();
  do
  {
  } while (ticY.getHomingActive() == 1);
}

void ZpololuGoHome()
{
  ticZ.goHomeReverse();
  do
  {
  } while (ticZ.getHomingActive() == 1);
  ticY.goHomeReverse();
}

//--------------------------------------//
// +++ pololu Arrive at set position +++//
//--------------------------------------//
void X_waitForPosition(int32_t targetPosition)
{
do
  {
    resetCommandTimeout();
  } while (ticX.getCurrentPosition() != targetPosition);
}

void Y_waitForPosition(int32_t targetPosition)
{
do
    {
    resetCommandTimeout();
  } while (ticY.getCurrentPosition() != targetPosition);
}

void Z_waitForPosition(int32_t targetPosition)
{
do
  {
    resetCommandTimeout();
  } while (ticZ.getCurrentPosition() != targetPosition);
}

//-------------------------------------//
//+++Nextion Command Trigger Receive+++//
//-------------------------------------//

//------------------------------------------------------------//
//+++START Push Nextion TouchReleaseEvent_Printh23 02 54 00+++//
//------------------------------------------------------------//
void trigger0()
{
  XpololuTargetPositionVer1();
  ZpololuTargetPositionVer1();
  YpololuTargetPositionVer1();
  ZpololuZeroPosition();
  XpololuTargetPositionVer2();
  ZpololuTargetPositionVer1();
  YpololuTargetPositionVer2();
  ZpololuZeroPosition();
  XpololuTargetPositionVer3();
  ZpololuTargetPositionVer1();
  YpololuTargetPositionVer3();
  ZpololuZeroPosition();
  XpololuTargetPositionVer4();
  ZpololuTargetPositionVer1();
  YpololuTargetPositionVer4();
  ZpololuZeroPosition();
}

//------------------------------------------------------------//
//+++Pause Push Nextion TouchReleaseEvent_Printh23 02 54 01+++//
//------------------------------------------------------------//
void trigger1()
{
  XpololuGoHome();
  YpololuGoHome();
  ZpololuGoHome();
}
//------------------------------------------------------------//
//+++Restart Push Nextion TouchReleaseEvent_Printh23 02 54 02+++//
//------------------------------------------------------------//
void trigger2()
{
  PosX = 20000;
  ticX.setTargetPosition(PosX);
  Pos16Y = 20000;
  ticY.setTargetPosition(Pos16Y);
  PosZ = 15000;
  ticZ.setTargetPosition(PosZ);
}

//------------------------------------------------------------//
//+++Reset Push Nextion TouchReleaseEvent_Printh23 02 54 03+++//
//------------------------------------------------------------//
void trigger3()
{
}

//------------------------------------------------------------//
//+++End Push Nextion TouchReleaseEvent_Printh23 02 54 04+++//
//------------------------------------------------------------//
void trigger4()
{
  ticX.exitSafeStart();
  delay(10);
  ticY.exitSafeStart();
  delay(10);
  ticZ.exitSafeStart();
  delay(10);
  ticX.energize();
  delay(10);
  ticY.energize();
  delay(10);
  ticZ.energize();
  delay(10);
}
